local function angledif(a1, a2)
    local dif = (a2 - a1) % (2*math.pi)
    if dif >= math.pi then
      return dif - 2*math.pi
    end
    return dif
end

local function hitpoint(px, py)
    -- Construct the quadratic equation.
    local a = mov1x^2 + mov1y^2
    local b = 2*(mov1x * (ball1x - px) + mov1y * (ball1y - py))
    local c = (ball1x - px)^2 + (ball1y - py)^2 - 10^2
    -- Discriminant
    local d = b^2 - 4*a*c
    if d >= 0 then
        -- Ball collides with corner. Find out where.
        d = math.sqrt(d)
        local t = (d - b) / (a * 2)
        do
            -- Second solution
            local t2 = (d + b) / (a * -2)
            -- Find the lowest positive one, if any
            -- Set t to the best candidate of both
            if t < 0 or t2 < t and t2 >= 0 then t = t2 end
        end
        if t >= 0 and t < 1 then
            -- We have a collision; calculate the point of
            -- collision with the ball
            local cx = ball1x + mov1x * t
            local cy = ball1y + mov1y * t

            -- Calculate the reflected vector.
            -- The normalized normal is (cx-px)/10, (cy-py)/10.
            local dot2 = 2 * ((cx - px) * mov1x + (cy - py) * mov1y)/10^2
            local rx = mov1x - (cx - px)*dot2
            local ry = mov1y - (cy - py)*dot2
            -- Now calculate the bounced position.
            -- It's a point that is at the same distance
            -- from cx,cy as is newball1x, newball1y from it,
            -- but in the direction rx, ry.
            local advance = math.sqrt((newball1x - cx)^2 + (newball1y - cy)^2)
            local rlen = math.sqrt(rx^2 + ry^2)
            newball1x = rx / rlen * advance + cx
            newball1y = ry / rlen * advance + cy
            mov1x = rx
            mov1y = ry
            if isfx then
                fx:play()
            end
            return true
        end
    end
    return false
end


local function handle_paddles_collision(ang)
    -- It must be out of the circle to start with, otherwise don't check 
    -- collisions (not too realistic, but it's not worth handling collisions
    -- with the inner sides of the paddles).
    local c = (ball1x - x)^2 + (ball1y - y)^2
    if c < 100^2 then return end

    -- Calculate the intersection points of the trajectory of the center
    -- with the Minkowski difference of the circles (which is another circle
    -- whose radius is the sum of both, i.e. 110 in this case)
    -- Construct the quadratic equation. We already have c.
    local a = mov1x^2 + mov1y^2
    local b = 2*(mov1x * (ball1x - x) + mov1y * (ball1y - y))
    c = c - 110^2
    -- Discriminant
    local d = b^2 - 4*a*c
    if d >= 0 then
        -- Collides with our circle. Find out where.
        d = math.sqrt(d)
        -- First solution
        local t = (d - b) / (a * 2)
        do
            -- Second solution
            local t2 = (d + b) / (a * -2)
            -- Find the lowest positive one, if any
            -- Set t to the best candidate of both
            if t < 0 or t2 < t and t2 >= 0 then t = t2 end
        end
        if t >= 0 and t < 1 then
            -- We have a collision; calculate the point of
            -- collision with the paddle circle
            local cx = ball1x + mov1x * t
            local cy = ball1y + mov1y * t
            -- Does it match either of the paddles?
            local in_range = false
            do
                local ball1ang = math.atan2(cy - y, cx - x)
                local padang = angledif(ang, ball1ang)
                if padang >= 0 and padang <= math.pi/4 then
                    in_range = true
                else
                    -- Check second paddle
                    padang = angledif(ang + math.pi, ball1ang)
                    if padang >= 0 and padang <= math.pi/4 then
                        in_range = true
                    end
                end
            end
            if in_range then
                -- Hit a paddle! Normal bounce.
                -- Calculate the reflected vector.
                -- The normalized normal is (cx-x)/110, (cy-y)/110.
                local dot2 = 2 * ((cx - x) * mov1x + (cy - y) * mov1y)/110^2
                local rx = mov1x - (cx - x)*dot2
                local ry = mov1y - (cy - y)*dot2
                -- Now calculate the bounced position.
                -- It's a point that is at the same distance
                -- from cx,cy as is newball1x, newball1y from it,
                -- but in the direction rx, ry.
                local advance = math.sqrt((newball1x - cx)^2 + (newball1y - cy)^2)
                local rlen = math.sqrt(rx^2 + ry^2)
                newball1x = rx / rlen * advance + cx
                newball1y = ry / rlen * advance + cy
                mov1x = rx
                mov1y = ry
                if isfx then
                    fx:play()
                end
                return
            end
        end
    end

    -- Corner case (literally!): collision with the endpoints of the paddles.
    return hitpoint(x+100*math.cos(ang)             , y+100*math.sin(ang))
      or hitpoint(x+100*math.cos(ang + math.pi*0.25), y+100*math.sin(ang + math.pi*0.25))
      or hitpoint(x+100*math.cos(ang + math.pi)     , y+100*math.sin(ang + math.pi))
      or hitpoint(x+100*math.cos(ang + math.pi*1.25), y+100*math.sin(ang + math.pi*1.25))
end

local function resolve_collsions()
    if (ball1y - y)^2 + (ball1x - x)^2 < 90^2 then
        -- Plain lost
        dead = true
        isgoing = false
        return
    end

    if newball1x < 10 then
        newball1x = 10 - (newball1x - 10)
        mov1x = -mov1x
        if isfx then
            fx:play()
        end
    elseif newball1x > 790 then
        newball1x = 790 - (newball1x - 790)
        mov1x = -mov1x
        if isfx then
            fx:play()
        end
    end
    if newball1y < 10 then
        newball1y = 10 - (newball1y - 10)
        mov1y = -mov1y
        if isfx then
            fx:play()
        end
    elseif newball1y > 590 then
        newball1y = 590 - (newball1y - 590)
        mov1y = -mov1y
        if isfx then
            fx:play()
        end
    end
    --if (newball1x - x)^2 + (newball1y - y)^2 < 110^2 then
        -- Ball reaches the range of the paddles in this frame
        handle_paddles_collision(angle)
    --end
end

function love.load()
    math.randomseed(os.time())
    math.random(); math.random(); math.random()
    angle = (3 * math.pi / 2) - (math.pi / 8)
    x = 400
    y = 300
    ball1x,ball1y = 30, 300
    speedint = 200
    speed1x,speed1y = speedint, math.random(-speedint,speedint)
    if speed1y < 50 and speed1y > 0 then
        speed1y = speed1y + 50
    end
    if speed1y > -50 and speed1y < 0 then
        speed1y = speed1y - 50
    end
    isgoing = false
    ismoving = false
    dead = false
    time = 0
    ball1angle = 0
    redangle,redangle2 = 0,0
    greenangle,greenangle2 = 0,0
    speedangle = 0
    isfx = true
    ismusic = true
    music = love.audio.newSource("sounds/techno80.mp3")
    music:setLooping(true)
    music:setVolume(60)
    music:play()
    fx = love.audio.newSource("sounds/ball.mp3")
    fx:setLooping(false)
    fx:setVolume(80)
end

function love.update(dt)
    ismoving = false
    if love.keyboard.isDown("left") then
        angle = (angle - (3*math.pi/4) * dt)
        if (angle <= 0) then
            angle = 2*math.pi
        end
        ismoving = true
    end
    if love.keyboard.isDown("right") then
        angle = (angle + (3*math.pi/4) * dt)
        if (angle >= 2 * math.pi) then
            angle = 0
        end
        ismoving = true
    end
    if not dead and isgoing then
        time = time + dt
        if math.ceil(time) % 10 == 0 then
            if speed1x > 0 then
                speed1x = speed1x + (50 * dt)
            else
                speed1x = speed1x - (50 * dt)
            end
        end
    end
    speedint = speedint + (1 * dt)
    if not dead and isgoing then
        mov1x = speed1x * dt
        mov1y = speed1y * dt
        newball1x = ball1x + mov1x
        newball1y = ball1y + mov1y
        resolve_collsions()
        speed1x = mov1x / dt
        speed1y = mov1y / dt
        ball1x = newball1x
        ball1y = newball1y
    end
    if love.keyboard.isDown("space") and not isgoing then
        isgoing = true
        ismoving = false
    end
    if love.keyboard.isDown("space") and dead then
        dead = false
        isgoing = true
        ismoving = false
        angle = (3 * math.pi / 2) - (math.pi / 8)
        x = 400
        y = 300
        ball1x,ball1y = 30, 300
        speedint = 200
        speed1x,speed1y = speedint, math.random(-speedint,speedint)
        if speed1y < 50 and speed1y > 0 then
            speed1y = speed1y + 50
        end
        if speed1y > -50 and speed1y < 0 then
            speed1y = speed1y - 50
        end
        time = 0
    end
    if love.keyboard.isDown("f5") then
        love.event.quit("restart")
    end
end

function love.keyreleased(key)
    if key == "escape" and not isgoing then
        love.event.quit()
    end
    if key == "escape" and isgoing then
        dead = false
        isgoing = false
        angle = (3 * math.pi / 2) - (math.pi / 8)
        x = 400
        y = 300
        ball1x,ball1y = 30, 300
        speedint = 200
        speed1x,speed1y = speedint, math.random(-speedint,speedint)
        if speed1y < 50 and speed1y > 0 then
            speed1y = speed1y + 50
        end
        if speed1y > -50 and speed1y < 0 then
            speed1y = speed1y - 50
        end
        time = 0
    end
    if key == "f" then
        if isfx then
            isfx = false
        else
            isfx = true
        end
    end
    if key == "m" then
        if ismusic then
            ismusic = false
            music:stop()
        else
            ismusic = true
            music:play()
        end
    end
end

function love.draw()
    --love.graphics.print(string.format("%.17g", math.sqrt(speed1x^2+speed1y^2)))
    love.graphics.setColor(255,0,0)
    love.graphics.arc("fill", "pie", x, y, 100, angle, (math.pi / 4) + angle)
    love.graphics.setColor(0,255,0)
    love.graphics.arc("fill", "pie", x, y, 100, angle - math.pi, (math.pi / 4) + angle - math.pi )
    love.graphics.setColor(0,0,255)
    love.graphics.circle("fill", x, y, 92)
    love.graphics.setColor(0,0,0)
    love.graphics.setLineWidth(8)
    love.graphics.circle("line", x, y, 92)
    love.graphics.setColor(255,0,255)
    love.graphics.circle("fill", ball1x, ball1y, 10)
    if not dead then
        if ismoving then
            love.graphics.setColor(255,255,0)
        else
            love.graphics.setColor(100,100,100)
        end
    else
        love.graphics.setColor(255,0,0)
    end
    love.graphics.circle("fill", x, y, 10)
    love.graphics.setColor(255,255,255)
    if not isgoing and not dead then
        love.graphics.setNewFont(24)
        love.graphics.print("Press Space to start.", 30, 30)
    end
    if not dead and isgoing then
        love.graphics.setNewFont(48)
        love.graphics.print(math.floor(time), 30, 30)
    end
    if dead and not isgoing then
        love.graphics.setNewFont(24)
        love.graphics.print("You survived "..math.floor(time).." seconds.", 30, 30)
        if time > 70 and time < 80 then
            love.graphics.print("Not bad!", 30, 90)
        elseif time > 80 and time < 85 then
            love.graphics.print("Awesome!", 30, 90)
        elseif time > 90 then
            love.graphics.print("You played just wonderfully!", 30 ,90)
        end
        love.graphics.print("Press Space to try again.", 30, 120)
    end
    love.graphics.setNewFont(16)
    love.graphics.setColor(255,255,255)
    if isfx then
        love.graphics.print("(F)x: on ", 600, 30)
    else
        love.graphics.print("(F)x: off", 600, 30)
    end
    if ismusic then
        love.graphics.print("(M)usic: on ", 680, 30)
    else
        love.graphics.print("(M)usic: off", 680, 30)
    end
end
